"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const _ = require('lodash');
const axios = require('axios').default;
const querystring = require('querystring');
class SwarmappApi {
    constructor(oauth_token) {
        this.config = {
            oauth_token: oauth_token,
            m: 'swarm',
            v: '20221101',
            // A random coordinate to use between calls imitating regular behavior
            ll: '26.30' + this.between(340000000000, 499999999999) + ',50.1' + this.between(2870000000000, 3119999999999),
            altAcc: "30.000000",
            llAcc: "14.825392"
        };
        this.basePath = 'https://api.foursquare.com/v2/';
        this.headers = {
            'User-Agent': 'com.foursquare.robin.ios.phone:20230316.2230.52:20221101:iOS 16.1.1:iPhone13,4'
        };
        this.initialize();
    }
    initialize() {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield this.getUser();
                this.user = (_b = (_a = response === null || response === void 0 ? void 0 : response.data) === null || _a === void 0 ? void 0 : _a.response) === null || _b === void 0 ? void 0 : _b.user;
                this.log("hello");
            }
            catch (error) {
                this.error("Could not authenticate user");
            }
        });
    }
    log(message) {
        var _a, _b;
        console.log(`${new Date().toLocaleString()} - ${(_a = this === null || this === void 0 ? void 0 : this.user) === null || _a === void 0 ? void 0 : _a.firstName}(${(_b = this === null || this === void 0 ? void 0 : this.user) === null || _b === void 0 ? void 0 : _b.id}) - `, message);
    }
    error(message) {
        var _a, _b;
        console.error(`${new Date().toLocaleString()} - ${(_a = this === null || this === void 0 ? void 0 : this.user) === null || _a === void 0 ? void 0 : _a.firstName}(${(_b = this === null || this === void 0 ? void 0 : this.user) === null || _b === void 0 ? void 0 : _b.id}) - Error:`, message);
    }
    initiatemultifactorlogin(username, password, client_id, client_secret) {
        return __awaiter(this, void 0, void 0, function* () {
            const params = {
                client_id: client_id,
                client_secret: client_secret,
                username: username,
                password: password,
            };
            try {
                const response = yield axios.post(this.basePath + '/private/initiatemultifactorlogin', null, { params });
                this.flowId = response.data.flowId;
                return response.data.access_token;
            }
            catch (error) {
                this.error(error);
                return;
            }
        });
    }
    completemultifactorlogin(code, client_id, client_secret) {
        return __awaiter(this, void 0, void 0, function* () {
            const params = {
                client_id: client_id,
                client_secret: client_secret,
                code: code,
                flowId: this.flowId
            };
            try {
                const response = yield axios.post(this.basePath + '/private/completemultifactorlogin', null, { params });
                this.config.oauth_token = response.data.oauth_token;
                return response.data.access_token;
            }
            catch (error) {
                this.error(error);
                return;
            }
        });
    }
    // Get Commands
    // TODO: requires testing, does not accept non swarmapp (mobile) oauth tokens
    getFriends(user_id = 'self') {
        return __awaiter(this, void 0, void 0, function* () {
            this.config.user_id = user_id;
            try {
                const result = yield axios.get(this.basePath + 'users/' + user_id + '/friends', { 'params': this.config });
                return result.data.response.friends;
            }
            catch (error) {
                this.error(error);
            }
        });
    }
    // TODO: works only for foursquare client, requires more testing
    getFollowings(user_id = 'self') {
        return __awaiter(this, void 0, void 0, function* () {
            this.config.m = 'foursquare';
            this.config.user_id = user_id;
            try {
                const result = yield axios.get(this.basePath + 'users/' + user_id + '/following', { 'params': this.config });
                return result.data.response.following;
            }
            catch (error) {
                this.error(error);
            }
        });
    }
    // TODO: works only for foursquare client, requires more testing
    getFollowers(user_id = 'self') {
        return __awaiter(this, void 0, void 0, function* () {
            this.config.m = 'foursquare';
            this.config.user_id = user_id;
            try {
                const result = yield axios.get(this.basePath + 'users/' + user_id + '/followers', { 'params': this.config });
                return result.data.response.followers;
            }
            catch (error) {
                this.error(error);
            }
        });
    }
    // TODO: should get user, then get checkins and location of last check-in
    getLastSeen(user_id = 'self', limit = 100) {
        this.config.user_id = user_id;
        this.config.limit = limit;
        try {
            return this.getUser(user_id);
        }
        catch (error) {
            this.error(error);
        }
    }
    getUser(user_id = 'self') {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield axios.get(`${this.basePath}users/${user_id}`, { 'params': this.config });
                return result;
            }
            catch (error) {
                throw new Error("Error getting user data, maybe an authentication error ?");
                return;
            }
        });
    }
    getCheckins(user_id = 'self', limit = 100, afterTimestamp) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof afterTimestamp !== 'undefined') {
                this.config.afterTimeStamp = afterTimestamp;
            }
            this.config.user_id = user_id;
            this.config.limit = limit;
            try {
                const result = yield axios.get(this.basePath + 'users/' + user_id + '/checkins', { 'params': this.config });
                return result;
            }
            catch (error) {
                this.error(error);
                return;
            }
        });
    }
    // not sure if it is working
    getGeos(user_id = 'self') {
        return __awaiter(this, void 0, void 0, function* () {
            this.config.user_id = user_id;
            delete this.config.afterTimeStamp;
            try {
                const response = yield axios.get(this.basePath + '/users/' + user_id + '/map', { 'params': this.config });
                return response.data.response;
            }
            catch (error) {
                this.error(error);
            }
        });
    }
    // returns user timeline after timestamp
    getRecent(limit = 100, ll) {
        return __awaiter(this, void 0, void 0, function* () {
            this.config.limit = limit;
            this.config.afterTimeStamp = (Math.floor(Date.now() / 1000) - (1 * 24 * 60 * 60)).toString();
            if (typeof ll !== 'undefined') {
                this.log('found location');
                this.config.ll = ll;
            }
            try {
                const result = yield axios.get(this.basePath + 'checkins/recent', {
                    params: this.config,
                    paramsSerializer: (params) => {
                        return querystring.stringify(params);
                    }
                });
                return result.data.response.recent;
            }
            catch (error) {
                this.error(error);
                return;
            }
        });
    }
    // Checkin Functions
    checkIn(location_id, silent) {
        return __awaiter(this, void 0, void 0, function* () {
            if (silent) {
                this.config.broadcast = 'private';
            }
            this.config.venueId = location_id;
            try {
                const result = yield axios.post(this.basePath + 'checkins/add', querystring.stringify(this.config));
                return result;
            }
            catch (error) {
                this.error(error);
                return;
            }
        });
    }
    addHereNow(checkin, females_only = true) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const hereNow = (_a = checkin === null || checkin === void 0 ? void 0 : checkin.venue) === null || _a === void 0 ? void 0 : _a.hereNow;
            this.log(`Adding hereNows from ${checkin.venue.name}`);
            this.log(`Females only setting is ${females_only}`);
            this.log(`Found ${hereNow.count} people checked in classified under ${hereNow.groups.length} groups`);
            if (hereNow.count > 0) {
                for (const group of hereNow.groups) {
                    this.log(`Found ${group.count} in group ${group.name}`);
                    if (group.count > 0) {
                        for (const item of group.items) {
                            if (females_only) {
                                if (((_b = item.user) === null || _b === void 0 ? void 0 : _b.gender) == 'female') {
                                    yield this.addFriendByID(item.user.id);
                                }
                            }
                            else {
                                yield this.addFriendByID(item.user.id);
                            }
                        }
                    }
                }
            }
            else {
                return 'no one is here';
            }
        });
    }
    addFriendByID(user_id) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield this.getUser(user_id.toString());
                const new_friend = (_b = (_a = result === null || result === void 0 ? void 0 : result.data) === null || _a === void 0 ? void 0 : _a.response) === null || _b === void 0 ? void 0 : _b.user;
                this.log(`Checking ${new_friend === null || new_friend === void 0 ? void 0 : new_friend.firstName} ${new_friend === null || new_friend === void 0 ? void 0 : new_friend.lastName}`);
                //console.log(new_friend);
                this.log(`Relationship status is ${new_friend.relationship}`);
                if (new_friend.relationship == 'none') {
                    const result = yield axios.post(this.basePath + 'users/' + user_id + '/request', querystring.stringify(this.config));
                    this.log(`Added ${new_friend === null || new_friend === void 0 ? void 0 : new_friend.firstName} ${new_friend === null || new_friend === void 0 ? void 0 : new_friend.lastName}`);
                    return result.data.response.user.relationship;
                }
            }
            catch (error) {
                this.error(error);
                return;
            }
        });
    }
    // Like Functions
    likeCheckin(checkin_id) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield axios.post(this.basePath + 'checkins/' + checkin_id + '/like', querystring.stringify(this.config));
                return result;
            }
            catch (error) {
                this.error(error);
                return;
            }
        });
    }
    likeUnliked(limit = 40) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const succeeded = [];
                const recent = yield this.getRecent(limit);
                for (const checkin of recent) {
                    this.log(`Checkin ${checkin.id} liked before = ${checkin.like}`);
                    if (checkin.like == false) {
                        const liked_result = yield this.likeCheckin(checkin.id);
                        this.log(`liked ${checkin.id}`);
                        succeeded.push(liked_result);
                    }
                }
                return succeeded;
            }
            catch (error) {
                this.error(error);
            }
        });
    }
    // Utility functions
    between(min, max) {
        return Math.floor(Math.random() * (max - min) + min);
    }
}
module.exports = SwarmappApi;
